' BAKCOPY.VBS       v.7     10-2019

Option Explicit

' set TRIAL to True for script testing without real file/folder copy or creation
Const TRIAL = False

' exit (error) codes
Const ERR_ABORT = 1
Const ERR_PARAMETER = 2
Const ERR_TARGETDRV = 3
Const ERR_INIFILE = 4
Const ERR_TARGETDIR = 5
Const ERR_COPY = 6

Dim MyTitle, Profile
Dim obj, var, fso, wsh

' parameters
Dim sDescription, sCopyCommand, sDestDrive, sDestFolder
Dim xCopyQuotes, xCreateFolders, xDriveInfo, xNoDriveList, xExitOnError
Dim xLogResult, xLogEcho, xLogFile, sLogFile, xLogAppend
Dim xProgressWin, sProgressMsg, xProgressDtm

' copy command & output strings
Dim sCopyItem, sQuote, sDelimiter, sTarget, sCommand, sErr, sLog, sLogMsg

' execution control
Dim xError, nFolders, n

MyTitle = "Backup Copy (BAKCOPY.VBS)"

' make profile path
If WScript.Arguments.Count Then
    Profile = WScript.Arguments(0)
Else
    Set obj = WScript.CreateObject("WScript.Shell")
    Profile = FullPath(obj.CurrentDirectory) & "bakcopy.ini"
    Set obj = Nothing
End If
If Not Exists(Profile) Then
    MsgBox "Profile (.ini file) " & Profile & " not found!", vbOKOnly + vbCritical, MyTitle
    WScript.Quit (ERR_INIFILE)
End If

' read parameters (profile values)
Set obj = CreateObject("VTool.Profile")
With obj
    .File = Profile
    sCopyCommand = .GetValue("copy", "command")
    xCopyQuotes = CNum(.GetValue("copy", "quotes"))
    xLogResult = CNum(.GetValue("copy", "resultlog"))
    xLogEcho = CNum(.GetValue("copy", "resultmsg"))
    xExitOnError = CNum(.GetValue("copy", "exitonerror"))
    var = .GetValue("copy", "copytitle")
    If var <> "" Then MyTitle = var
    sDescription = .GetValue("copy", "description")
    If sDescription <> "" Then sDescription = sDescription & vbNewLine
    sDestDrive = .GetValue("destination", "defaultdrive")
    sDestFolder = .GetValue("destination", "copydir")
    xCreateFolders = CNum(.GetValue("destination", "createfolders"))
    xDriveInfo = CNum(.GetValue("destination", "driveinfo"))
    xNoDriveList = CNum(.GetValue("destination", "nodrivelist"))
    xLogFile = CNum(.GetValue("log", "savelog"))
    sLogFile = .GetValue("log", "logfile")
    xLogAppend = CNum(.GetValue("log", "appendlog"))
    xProgressWin = CNum(.GetValue("options", "progresswin"))
    xProgressDtm = CNum(.GetValue("options", "progressdtm"))
    sProgressMsg = .GetValue("options", "progresstxt")
    var = .GetSection("folders")
End With
Set obj = Nothing

If sCopyCommand = "" Then
    MsgBox "No copy command specified!", vbOKOnly + vbCritical, MyTitle
    WScript.Quit (ERR_PARAMETER)
End If
If var = "" Then
    MsgBox "No source folder specified!", vbOKOnly + vbCritical, MyTitle
    WScript.Quit (ERR_PARAMETER)
End If

sCopyItem = Split(var, vbNewLine)

' drive selection/check
If xNoDriveList Then
    If sDestDrive = "" Then
        MsgBox "No target drive defined!" & vbNewLine, vbOKOnly + vbCritical, MyTitle
        WScript.Quit (ERR_TARGETDRV)
    End If
End If
var = MsgBox(sDescription & "Next step will be to check/select the drive used for copying." & _
        vbNewLine & "If you want to use an external device (like USB stick or disk), " & _
        " then please connect it NOW!", vbOKCancel + vbExclamation, MyTitle)
If var = vbCancel Then
    WScript.Quit (ERR_ABORT)
End If

If xNoDriveList Then
    var = sDestDrive
Else
        ' <1>
    Set obj = CreateObject("VTool.DriveList")
    var = obj.Selection(CStr(sDestDrive))
        ' </1> <2>
    'Set obj = CreateObject("VTool.List")
    'var = obj.Selection(DriveList2, 2, "Drive selection")  ' 2 ~ D:
        ' </2> <3>
    'Set obj = CreateObject("VTool.List")
    'var = obj.Selection(DriveList3, , "Backup Destination")
        ' </3>
    Set obj = Nothing
    If var = "" Then
        MsgBox "No drive selected!" & vbNewLine, vbOKOnly + vbCritical, MyTitle
        WScript.Quit (ERR_TARGETDRV)
    End If
End If

Set fso = CreateObject("Scripting.FileSystemObject")
        ' fso is shared by subsequent functions

Set obj = ProperDrive(var)
If obj Is Nothing Then
    MsgBox "The chosen drive " & var & " is not ready (not accessible)!" & vbNewLine & _
        "Please check the cause. The program will terminate now." & _
        vbNewLine, vbOKOnly + vbCritical, MyTitle
    WScript.Quit (ERR_TARGETDRV)
End If

' complete destination path
If sDestFolder = "" Then
    sDestFolder = var & "\"
ElseIf Left(sDestFolder, 1) = "\" Then
    sDestFolder = var & sDestFolder
Else
    sDestFolder = var & "\" & sDestFolder
End If

' start copy process?
var = sDescription & "Next step will start the copy process." & vbNewLine & _
        "Copy target is:" & vbNewLine & UCASE(sDestFolder) & vbNewLine
If xDriveInfo Then
    var = var & vbNewLine & "drive type = " & CStr(GetDriveType(obj)) & vbNewLine & _
        "available space (ca.) = " & CStr(GetDriveSpace(obj)) & vbNewLine
End If
var = MsgBox(var, vbOKCancel + vbExclamation, MyTitle)
If var = vbCancel Then
    WScript.Quit (ERR_ABORT)
End If

If Not TRIAL Then
    If Not FolderIsThere(sDestFolder) Then
        MsgBox "The destination folder " & sDestFolder & _
            " does not exist and could not be created!" & vbNewLine & _
            "Please check the cause. The program will terminate now." & _
            vbNewLine, vbOKOnly + vbCritical, MyTitle
        WScript.Quit (ERR_TARGETDIR)
    End If
End If

Set wsh = WScript.CreateObject("WScript.Shell")
        ' wsh is shared by subsequent functions

' prepare logfile
If xLogFile Then
    If Len(sLogFile) = 0 Then
        sLogFile = FullPath(wsh.CurrentDirectory) & "bakcopy.log"
    End If
    If SetupLogfile(sLogFile, xLogAppend) Then
        WriteLog vbNewLine & vbNewLine
        WriteLog "Backup: " & MyTitle & vbNewLine & sDescription
        WriteLog "START: " & CStr(Now)
        WriteLog String(50, "=") & vbNewLine
    Else
        xLogFile = False
    End If
End If

' copy process
sQuote = Chr(34)
If xCopyQuotes Then
    sDelimiter = sQuote
Else
    sDelimiter = ""
End If

nFolders = UBound(sCopyItem) - 1    ' Split returns an additional empty element!

If xProgressWin Then Set obj = OpenProgressWindow(nFolders)

For n = 0 To nFolders
    xError = 0
    sErr = ""
    sCopyItem(n) = wsh.ExpandEnvironmentStrings(sCopyItem(n))

    If xProgressWin Then
        obj.SubText = sCopyItem(n)
        If xProgressDtm Then obj.ProgressStep
    End If

    If xCreateFolders Then
        sTarget = FullPath(sDestFolder) & fso.GetBaseName(sCopyItem(n))
        If Not TRIAL Then
            If Not FolderIsThere(sTarget) Then
                xError = 1
                sErr = " /ERROR: Could not either find or create target folder: " & sTarget
            End If
        End If
    Else
        sTarget = sDestFolder
    End If
    sCommand = Replace(sCopyCommand, "[[SOURCE]]", sDelimiter & sCopyItem(n) & sDelimiter)
    sCommand = Replace(sCommand, "[[DEST]]", sDelimiter & sTarget & sDelimiter)

    If xLogFile Then WriteLog "COPY CMD " & CStr(n + 1) & ": " & sCommand

    If Not TRIAL Then
        If xLogFile Then
            sCommand = sCommand & " >>" & sDelimiter & sLogFile & sDelimiter & " 2>&1"
            var = RunCommand(sCommand, False)
        Else
            var = RunCommand(sCommand, True)
        End If
    End If
    sLog = MyTitle & ": " & CStr(Now) & " - Copy command " & CStr(n + 1) & " (" & _
            sCommand & ") returns " & CStr(var) & "!" & sErr

    If xLogFile Then
        WriteLog "== " & sLog & vbNewLine
    End If
    If xLogResult Then
        wsh.LogEvent 4, sLog
    End If
    If xLogEcho Then
        sLogMsg = sLogMsg & (sLog & vbNewLine)
    End If

    If xError Then
        ' consider additional actions
        If xExitOnError Then Exit For
    End If
Next

If xLogFile Then
    WriteLog "ENDING: " & CStr(Now)
    WriteLog String(50, "=") & vbNewLine
End If

If xProgressWin Then obj.Remove

Set wsh = Nothing
Set fso = Nothing

' show final message
If xError Then
    var = sDescription & "Backup copy process terminated. (Error)" & vbNewLine
    n = vbOKOnly + vbCritical
Else
    var = sDescription & "Backup copy process finished." & vbNewLine
    n = vbOKOnly + vbInformation
End If
If xLogEcho Then var = var & sLogMsg

Set obj = WScript.CreateObject("WScript.Shell")
obj.Popup var, , MyTitle, n
Set obj = Nothing

If xError Then WScript.Quit (ERR_COPY)

WScript.Quit (0)


' CNum - check/convert numeric values (INI file)
Function CNum(x)
    If IsNumeric(x) Then
        CNum = CInt(x)
    Else
        CNum = 0
    End If
End Function

' DriveList2 - alternative drive selection (commented out)
Function DriveList2
    Dim fs, dc, d, s, v
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set dc = fs.Drives
    For Each d in dc
        v = ""
        s = s & d.DriveLetter & ": - "
        If d.DriveType = 3 Then
            v = d.ShareName
        ElseIf d.IsReady Then
            v = d.VolumeName
        End If
        s = s & v & vbNewLine
    Next
    DriveList2 = s
End Function

' DriveList3 - alternative drive selection (commented out)
'   this version returns only drives that are ready
Function DriveList3
    Dim fs, dc, d, s, v
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set dc = fs.Drives
    For Each d in dc
        v = ""
        If d.IsReady Then
            If d.DriveType = 3 Then
                v = d.ShareName
            Else
                v = d.VolumeName
            End If
            If Len(v) Then v = " [" & v & "]"
            s = s & d.DriveLetter & ":" & v & vbNewLine
        End If
    Next
    DriveList3 = s
End Function

' Exists - check file presence (INI file)
Function Exists(F)
    Dim fs
    Set fs = CreateObject("Scripting.FileSystemObject")
    ' API functions require a full path to the file
    F = fs.GetAbsolutePathName(F)
    Exists = fs.FileExists(F)
End Function

' FolderIsThere - check/create folder (backup folder, target folders)
Function FolderIsThere(ByVal sfolder)
    'shared fso !
    On Error Resume Next
    If Not (fso.FolderExists(sfolder)) Then fso.CreateFolder(sfolder)
    If Err Then
        FolderIsThere = False
        Err.Clear
    Else
        FolderIsThere = True
    End If
    On Error GoTo 0
End Function

' FullPath - create proper path (INI file, backup/target folders, logfile)
Function FullPath(ByVal P)
    P = RTrim(P)
    If P <> "" Then
        If Right(P, 1) <> "\" Then P = P + "\"
    End If
    FullPath = P
End Function

' GetDriveSpace - return target drive information (message)
Function GetDriveSpace(d)
    GetDriveSpace = FormatNumber(d.AvailableSpace/1048576, 1) & " MB"
End Function

' GetDriveType - return target drive information (message)
Function GetDriveType(d)
    Select Case d.DriveType
        Case 1: GetDriveType = "Removable"
        Case 2: GetDriveType = "Fixed (e.g. hard disk)"
        Case 3: GetDriveType = "Network (LAN storage medium)"
        Case 4: GetDriveType = "CD-ROM (optical disc)"
        Case 5: GetDriveType = "RAM Disk (virtual drive)"
        Case Else: GetDriveType = "Unknown"
    End Select
End Function

' OpenProgressWindow - create/return optional progress bar window
Function OpenProgressWindow(nf)
    'shared MyTitle, sProgressMsg, xProgressDtm, xProgressWin !
    Dim pw
    On Error Resume Next
    Set pw = CreateObject("VTool.ProgressWindow")
    With pw
        .Title = MyTitle
        .MainText = sProgressMsg
        If xProgressDtm Then
            '.ProgressStyle = 0  ' default
            .SetProgressRange 0, nf + 1
            .SetProgressStep 1
        Else
            .ProgressStyle = 8
        End If
        .Create
    End With
    If Err Then
        Err.Clear
        xProgressWin = False
        Set OpenProgressWindow = Nothing
    Else
        Set OpenProgressWindow = pw
    End If
    On Error GoTo 0
End Function

' ProperDrive - check/return target drive specification (message, backup/target folders)
Function ProperDrive(D)
    'shared fso !
    Dim drv
    On Error Resume Next
    Set ProperDrive = Nothing
    If Not VarType(D) = vbString Then D = "" : Exit Function
    Select Case Len(D)
        Case 0
            Exit Function
        Case 1
            D = D & ":"
        Case 2
            ' D = D
        Case Else
            D = Left(D, 2)
    End Select
    Set drv = fso.GetDrive(D)
    If drv.IsReady Then Set ProperDrive = drv
    If Err Then Err.Clear
    On Error GoTo 0
End Function

' RunCommand - execute the copy command
Function RunCommand(cmdline, simple)
    'shared wsh (WScript.Shell), xError, sErr !
    On Error Resume Next
    If simple Then
        RunCommand = wsh.Run(cmdline, , vbTrue)
    Else
        RunCommand = wsh.Run("cmd.exe /C " & cmdline, , vbTrue)
    End If
    If Err Then
        xError = 2
        sErr = sErr & " /ERROR: " & Err.Description & " (#" & CStr(Err.Number) & ")"
        Err.Clear
    End If
    On Error GoTo 0
End Function

' SetupLogfile - check/prepare logfile
Function SetupLogfile(sfile, append)
    'shared fso !
    Dim otf, mode
    On Error Resume Next
    If append Then
        mode = 8    ' ForAppending
    Else
        mode = 2    ' ForWriting
    End If
    Set otf = fso.OpenTextFile(sfile, mode, True)   ' open/create file
    otf.Close
    If Err Then
        Err.Clear
        SetupLogfile = False
    Else
        SetupLogfile = True
    End If
    On Error GoTo 0
End Function

' WriteLog - write text to logfile
Sub WriteLog(record)
    'shared fso, sLogFile, xLogFile !
    Dim otf
    On Error Resume Next
    Set otf = fso.OpenTextFile(sLogFile, 8, True)   ' open/create for appending
    otf.WriteLine record
    otf.Close
    If Err Then
        Err.Clear
        xLogFile = 0
    End If
    On Error GoTo 0
End Sub

